from UserManager import UserManager
from emoji_test_runner import EmojiTestRunner
from user_models import *
from pymongo.errors import DuplicateKeyError
#from pydantic_core._pydantic_core import ValidationError
import unittest
import logging

# Set up a logger
logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(message)s")

conn_str = "mongodb://localhost:27017/"
um = UserManager(conn_str)

class TestUserManager(unittest.TestCase):

    def setUp(self):
        ''' Run before each test '''
        logger.info("\nStarting a new test...")
        um.reset()  # Ensure the DB is reset before each test
        logger.info("Database reset complete.")

    def tearDown(self):
        ''' Run after each test '''
        logger.info("Test completed.\n")

    def test_basic(self):
        ''' reset, create user, read user by id'''
        logger.info(f"Running \033[1m\033[38;5;5mtest_basic\033[0m")
        # # reset
        # um.reset()

        # create a new user
        userData = UserIn(username= "ta_user", password= "secret")
        createUserRes = um.create_user(userData)
        logger.info(f"New user created with id: {createUserRes.inserted_id}")

        # If the user was created we 
        # should be able to read it by the id
        newUser = um.read_user(createUserRes.inserted_id)

        # validate user was created
        self.assertIsNotNone(newUser)
        logger.info(f"Run Complete \033[1m\033[38;5;6mtest_basic\033[0m")


    @unittest.skip
    def test_notfound(self):
        ''' try to read user with invalid id, assert no user'''
         # reset
        # um.reset()
        logger.info(f"Running \033[1m\033[38;5;5mtest_notfound\033[0m")
        # invalid user data
        invalidUserId = "nonsense"

        # try to read an invalid user
        invalidUser = um.read_user(invalidUserId)

        # expect that no user is found (none)
        self.assertIsNone(invalidUser)
        logger.info(f"Run Complete \033[1m\033[38;5;6mtest_notfound\033[0m")

    def test_unique(self):
        ''' reset, create twice with same username '''
         # reset
        # um.reset()
        logger.info(f"Running \033[1m\033[38;5;5mtest_unique\033[0m")
         # Create user once
        dupUserDict = {"username": "dup_user", "password": "duplicate"}
        dupUserData = UserIn(**dupUserDict)
         # Create user and get the result
        createUserRes = um.create_user(dupUserData)
         # Assert that the first user was created successfully
        self.assertEqual(createUserRes.status, 1) 
        
        # Try to create the same user again and assert that the result contains the correct status and message
        createUserRes = um.create_user(dupUserData)
        
        # Assert that the status is 0 (duplicate)
        self.assertEqual(createUserRes.status, 0)
        # Assert that the message indicates the username is already taken
        self.assertEqual(createUserRes.message, "Sorry, username is alredy taken.")
        logger.info(f"Run Complete \033[1m\033[38;5;6mtest_unique\033[0m")
            

    def test_read_all(self):
        ''' reset, create many, read all, verify'''
         # reset
        # um.reset()
        logger.info(f"Running \033[1m\033[38;5;5mtest_read_all\033[0m")

        # Create many users
         
        users_to_create = [
            {"username": "user1", "password": "pass1"},
            {"username": "user2", "password": "pass2"},
            {"username": "user3", "password": "pass3"}
        ]

        # Create users
        for user_data in users_to_create:
            c_id = um.create_user(UserIn(**user_data)).inserted_id
            logger.info(f"User created with id: {c_id}")

        # verify
        all_users = um.read_users({})

        #print(f"\n Printing all_users from test_read_all test: {all_users}\n")

         # Verify the length of the users in the UserCollection
        self.assertEqual(len(all_users.users), len(users_to_create), "Number of created users doesn't match.")
        # Verify each created user is in the result
        for user_data in users_to_create:
            # Assert that each user created is found in the list of users returned
            self.assertTrue(
                any(user.username == user_data['username'] for user in all_users.users)
            )
        logger.info(f"Run Complete \033[1m\033[38;5;6mtest_read_all\033[0m")

    def test_set_address(self):
        ''' reset, create user, set address, read and verify address set'''
        logger.info(f"Running \033[1m\033[38;5;5mtest_set_address\033[0m")
        userData = {"username": "coolUserToUpdate", "password": "coolPass"}
        userIn = UserIn(**userData)
        user = um.create_user(userIn)

        updateWithAdd = {"address": {"line1": "64 Zoo Lane", "city": "Boston", "state": "Massachusetts", "zip": "12345"}}
        updateResult = um.update_user(user.inserted_id, updateWithAdd)
        self.assertEqual(updateResult.modified_count, 1)

        # verify adddress was updated
        updatedUser = um.read_user(user.inserted_id)
        self.assertEqual(updatedUser.address.city, "Boston")
        logger.info(f"Run Complete \033[1m\033[38;5;6mtest_set_address\033[0m")
        
    def test_delete(self):
        ''' reset, create user, read user, assert no user'''
        logger.info(f"Running \033[1m\033[38;5;5mtest_delete\033[0m")
        # create user
        userData = {"username": "userToDelete", "password": "deleteMe"}
        newUser = um.create_user(UserIn(**userData))

        # read user
        user = um.read_user(newUser.inserted_id)

        # delete user
        deletedResult = um.delete_user(user.id)
        self.assertEqual(deletedResult.delete_count, 1)
        self.assertIsNone(um.read_user(newUser.inserted_id))
        logger.info(f"Run Complete \033[1m\033[38;5;6mtest_delete\033[0m")

if __name__=="__main__":
    #unittest.main(verbosity=2)
    # Running tests with EmojiTestRunner to show emojis
    unittest.main(testRunner=EmojiTestRunner(), verbosity=2)



